from pymol import cmd, cgo
from pymol.cgo import *
# 全局变量，用于跟踪向量数量
vector_counter = 1

def draw_vector(x1, y1, z1, x2, y2, z2):
    global vector_counter  # 引入全局计数器
    
    # Convert the arguments to floats to ensure they are real numbers
    x1, y1, z1, x2, y2, z2 = float(x1), float(y1), float(z1), float(x2), float(y2), float(z2)
    
    # 定义箭头头部的大小
    arrow_radius = 0.3 # 箭头底部的半径
    arrow_length = 0.5  # 箭头的长度
    cylinder_radius = 0.1  # 圆柱的半径

    # 计算箭头起点 (箭头的尾部，也就是圆柱的末端)
    vec_x = x2 - x1
    vec_y = y2 - y1
    vec_z = z2 - z1
    norm = (vec_x**2 + vec_y**2 + vec_z**2)**0.5
    
    # 箭头尾端位置 (让箭头稍微缩短一段)
    arrow_base_x = x2 - (arrow_length * vec_x / norm)
    arrow_base_y = y2 - (arrow_length * vec_y / norm)
    arrow_base_z = z2 - (arrow_length * vec_z / norm)


    print("coord1",x1,y1,z1)
    print("coord2",x2,y2,z2)
    print("middle",arrow_base_x, arrow_base_y, arrow_base_z)
    # 定义CGO对象（圆柱 + 箭头）
    vector_cgo = [


        # 箭头部分 (CONE)
    
        cgo.CONE, arrow_base_x, arrow_base_y, arrow_base_z, x2, y2, z2, arrow_radius, 0.0, 
        1.0, 0.0, 0.0, # 箭头底部颜色 (红色)
        1.0, 0.0, 0.0,# 箭头顶端颜色 (红色)
        1.0, 1.0, # 封闭圆锥底部
    
        # 圆柱体部分
    
        cgo.CYLINDER, x1, y1, z1, arrow_base_x, arrow_base_y, arrow_base_z, cylinder_radius, 
        1.0, 0.0, 0.0,  # 起点颜色 (红色)
        1.0, 0.0, 0.0,  # 终点颜色 (红色)
      
        # w = 0.06 # cylinder width
        # l = 0.75 # cylinder length
        # h = 0.25 # cone hight
        # d = w * 1.618 # cone base diameter
        # cgo.CONE,   l, 0.0, 0.0,        h+l, 0.0, 0.0,           d, 0.0, 
        # 1.0, 0.0, 0.0, 
        # 1.0, 0.0, 0.0, 
        # 1.0, 1.0,

    ]
    
    # 使用递增的名字
    obj_name = "vector_%03d" % vector_counter
    vector_counter += 1  # 递增计数器

    # 加载CGO对象
    cmd.load_cgo(vector_cgo, obj_name)

# 扩展 PyMOL 命令，使其支持 draw_vector
cmd.extend("draw_vector", draw_vector)

